import sys
import socket
import proxy
import tempfile
import shutil
import ipaddress
import platform
import os
import pathlib
import logging
import multiprocessing

## Package Imports
from . import certgen
from . import sysprox as sprox

class Proxverter:
    '''
    The main Proxverter class that accepts creds, setup system wide caches and run proxy servers.
    '''

    def __init__(self, ip, port, is_https=False, new_certs=False, sysprox=False, verbose=False, plugins=[]):
        self.ip_address = ip
        self.port       = port
        self.is_https   = is_https
        self.new_certs  = new_certs
        self.sysprox    = sysprox
        self.verbose    = verbose
        self.plugins    = plugins
        self.home_paths = self.__fetch_home_paths()
        self.proxy      = sprox.Proxy(self.ip_address, self.port)

        ## Generating necessary data for certificates and private key
        if self.is_https:
            self.__gen_certs()

        ## Setting system wide proxy
        if self.sysprox:
            self.set_sysprox()

        ## Setting verbose mode
        if not self.verbose:
            logging.disable(logging.CRITICAL)

    def __fetch_home_paths(self):
        dirname = os.path.join(pathlib.Path.home(), ".proxverter")
        rtval = {
            'dirname': dirname,
            'certname': os.path.join(dirname, "cert.pem"),
            'privname': os.path.join(dirname, "priv.pem"),
            'pfxname': os.path.join(dirname, "cert.pfx")
        }

        if not os.path.isdir(rtval['dirname']):
            os.makedirs(rtval['dirname'])

        return rtval

    def __gen_certs(self):
        if not os.path.isfile(self.home_paths['certname']) \
           or not os.path.isfile(self.home_paths['privname']) \
           or not os.path.isfile(self.home_paths['pfxname']) \
           or self.new_certs:

           gen = certgen.Generator()
           gen.generate()
           gen.gen_key(self.home_paths['privname'])
           gen.gen_cert(self.home_paths['certname'])
           gen.gen_pfx(self.home_paths['pfxname'])

    def __clear(self):
        try:
            shutil.rmtree(
                os.path.join(pathlib.Path.home(), ".proxy")
            )
        except FileNotFoundError:
            pass

    def set_sysprox(self):
        self.proxy.engage()

    def del_sysprox(self):
        self.proxy.cleanup()

    def fetch_pkey(self, destination):
        if not self.is_https:
            raise ValueError("Private keys are only implemented in SSL Mode")

        if not os.path.isfile(self.home_paths['privname']):
            raise FileNotFoundError("No private key file was found in home directory. It has either been modified or deleted. ")

        shutil.copyfile(self.home_paths['privname'], destination)

    def fetch_cert(self, destination):
        if not self.is_https:
            raise ValueError("Certificates are only implemented in SSL Mode")

        if not os.path.isfile(self.home_paths['certname']):
            raise FileNotFoundError("No cert file was found in home directory. It has either been modified or deleted. ")

        shutil.copyfile(self.home_paths['certname'], destination)

    def fetch_pfx(self, destination):
        if not self.is_https:
            raise ValueError("PFXs are only implemented in SSL Mode")

        if not os.path.isfile(self.home_paths['certname']):
            raise FileNotFoundError("No pfx file was found in home directory. It has either been modified or deleted. ")

        shutil.copyfile(self.home_paths['pfxname'], destination)

    def engage(self):
        multiprocessing.freeze_support()
        self.__clear()

        try:
            if not self.is_https:
                proxy.main(
                    hostname = ipaddress.IPv4Address(self.ip_address),
                    port = self.port,
                    plugins = self.plugins
                )
            else:
                if not os.path.isfile(self.home_paths['privname']):
                    raise FileNotFoundError("Given private key file doesn't exists")

                if not os.path.isfile(self.home_paths['certname']):
                    raise FileNotFoundError("Given certificate file doesn't exists")

                proxy.main(
                    hostname = ipaddress.IPv4Address(self.ip_address),
                    port = self.port,
                    ca_key_file = self.home_paths['privname'],
                    ca_cert_file = self.home_paths['certname'],
                    ca_signing_key_file = self.home_paths['privname'],
                    plugins = self.plugins
                )

        except KeyboardInterrupt:
            pass

        if self.sysprox:
            self.del_sysprox()



import sys
import os
import tempfile
import proxverter
import logging
import multiprocessing

if __name__ == "__main__":
    multiprocessing.freeze_support()

    p = proxverter.Proxverter(
        "127.0.0.1",
        8899,
        is_https=True,
        verbose=True
    )

    fl1 = os.path.join(tempfile.gettempdir(), 'priv.key')
    fl2 = os.path.join(tempfile.gettempdir(), 'cert.crt')
    fl3 = "certificate.pfx"

    #p.gen_key(fl1)
    #p.gen_cert(fl2)
    #p.gen_pfx(fl3)

    p.join(fl1, fl2)



from OpenSSL import crypto, SSL
import OpenSSL
import random

class Generator:

    def __init__(self, email=None, country=None, province=None, locality=None, organization=None, unit=None, commonname=None):
        self.email   = email
        self.country = country
        self.province = province
        self.locality = locality
        self.organization = organization
        self.unit     = unit
        self.commonname = commonname or "example.com"
        self.serial_number = random.getrandbits(64)
        self.valid_start = 0
        self.valid_end   = 10*365*24*60*60

    def generate(self):
        self.cert = crypto.X509()
        self.key = crypto.PKey()
        self.key.generate_key(crypto.TYPE_RSA, 2048)
        if self.country:
            self.cert.get_subject().C = self.country
        if self.province:
            self.cert.get_subject().ST = self.province
        if self.locality:
            self.cert.get_subject().L = self.locality
        if self.organization:
            self.cert.get_subject().O = self.organization
        if self.unit:
            self.cert.get_subject().OU = self.unit
        self.cert.get_subject().CN = self.commonname
        if self.email:
            self.cert.get_subject().emailAddress = self.email
        self.cert.set_serial_number(self.serial_number)
        self.cert.gmtime_adj_notBefore(self.valid_start)
        self.cert.gmtime_adj_notAfter(self.valid_end)
        self.cert.set_issuer(self.cert.get_subject())
        self.cert.set_pubkey(self.key)
        self.cert.sign(self.key, 'sha256')

    def gen_key(self, key_file):
        key_file = open(key_file, 'wt')
        try:
            key_file.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, self.key).decode("utf-8"))
        except TypeError:
            key_file.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, self.key))
        key_file.close()

    def gen_cert(self, cert_file):
        cert_file = open(cert_file, 'wt')
        try:
            cert_file.write(crypto.dump_certificate(crypto.FILETYPE_PEM, self.cert).decode("utf-8"))
        except TypeError:
            cert_file.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, self.key))
        cert_file.close()

    def gen_pfx(self, pfx_file):
        p12 = OpenSSL.crypto.PKCS12()
        p12.set_privatekey( self.key )
        p12.set_certificate( self.cert )

        pfx_file = open(pfx_file, 'wb')
        pfx_file.write(p12.export())
        pfx_file.close()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="proxverter",
    version="0.1",
    author="hash3liZer",
    author_email="admin@shellvoide.com",
    description="Reverse proxy & TLS Interception library for Python. Basically a wrapper around proxy.py and OpenSSL allowing easy integration and certificate generation on command (Under Development)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hash3liZer/Proxverter",
    project_urls={
        "Bug Tracker": "https://github.com/hash3liZer/Proxverter/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
        "Operating System :: OS Independent",
    ],
    packages=['proxverter', 'proxverter.test'],
    python_requires=">=3.7",
    install_requires=[
        'pyopenssl',
        'proxy.py'
    ]
)
